@@ -190,6 +190,8 @@ void
 arm_init(int when)
 {
 	ulong vaddr;
+	char *string;
+	struct syment *sp;
 
 #if defined(__i386__) || defined(__x86_64__)
 	if (ACTIVE())
@@ -229,8 +231,13 @@ arm_init(int when)
 		 * LPAE requires an additional page for the PGD, 
 		 * so PG_DIR_SIZE = 0x5000 for LPAE
 		 */
-		if ((symbol_value("_text") - symbol_value("swapper_pg_dir")) == 0x5000)
+		if ((string = pc->read_vmcoreinfo("CONFIG_ARM_LPAE"))) {
 			machdep->flags |= PAE;
+			free(string);
+		} else if ((sp = next_symbol("swapper_pg_dir", NULL)) &&
+		         (sp->value - symbol_value("swapper_pg_dir")) == 0x5000)
+                         machdep->flags |= PAE;
+
 		machdep->kvbase = symbol_value("_stext") & ~KVBASE_MASK;
 		machdep->identity_map_base = machdep->kvbase;
 		machdep->is_kvaddr = arm_is_kvaddr;
@@ -315,6 +322,9 @@ arm_init(int when)
 				   "pr_pid");
 		MEMBER_OFFSET_INIT(elf_prstatus_pr_reg, "elf_prstatus",
 				   "pr_reg");
+	
+		if (!machdep->hz)
+			machdep->hz = 100;
 		break;
 
 	case POST_VM:
@@ -570,8 +580,8 @@ arm_get_crash_notes(void)
 
 	buf = GETBUF(SIZE(note_buf));
 
-	if (!(panic_task_regs = malloc(kt->cpus*sizeof(*panic_task_regs))))
-		error(FATAL, "cannot malloc panic_task_regs space\n");
+	if (!(panic_task_regs = calloc((size_t)kt->cpus, sizeof(*panic_task_regs))))
+		error(FATAL, "cannot calloc panic_task_regs space\n");
 	
 	for  (i=0;i<kt->cpus;i++) {
 
@@ -587,6 +597,33 @@ arm_get_crash_notes(void)
 		note = (Elf32_Nhdr *)buf;
 		p = buf + sizeof(Elf32_Nhdr);
 
+		/*
+		 * dumpfiles created with qemu won't have crash_notes, but there will
+		 * be elf notes; dumpfiles created by kdump do not create notes for
+		 * offline cpus.
+		 */
+		if (note->n_namesz == 0 && (DISKDUMP_DUMPFILE() || KDUMP_DUMPFILE())) {
+			if (DISKDUMP_DUMPFILE())
+				note = diskdump_get_prstatus_percpu(i);
+			else if (KDUMP_DUMPFILE())
+				note = netdump_get_prstatus_percpu(i);
+			if (note) {
+				/*
+				 * SIZE(note_buf) accounts for a "final note", which is a
+				 * trailing empty elf note header.
+				 */
+				long notesz = SIZE(note_buf) - sizeof(Elf32_Nhdr);
+
+				if (sizeof(Elf32_Nhdr) + roundup(note->n_namesz, 4) +
+				    note->n_descsz == notesz)
+					BCOPY((char *)note, buf, notesz);
+			} else {
+				error(WARNING,
+					"cannot find NT_PRSTATUS note for cpu: %d\n", i);
+				continue;
+			}
+		}
+
 		if (note->n_type != NT_PRSTATUS) {
 			error(WARNING, "invalid note (n_type != NT_PRSTATUS)\n");
 			goto fail;
@@ -825,6 +862,9 @@ arm_back_trace(struct bt_info *bt)
 static void
 arm_back_trace_cmd(struct bt_info *bt)
 {
+	if (bt->flags & BT_REGS_NOT_FOUND)
+		return;
+
 	if (kt->flags & DWARF_UNWIND)
 		unwind_backtrace(bt);
 	else
@@ -1085,6 +1125,18 @@ arm_lpae_vtop(ulong vaddr, ulong *pgd, physaddr_t *paddr, int verbose)
 	pmd_t pmd_pte;
 	pte_t pte;
 
+	if (!vt->vmalloc_start) {
+		*paddr = LPAE_VTOP(vaddr);
+		return TRUE;
+	}
+
+	if (!IS_VMALLOC_ADDR(vaddr)) {
+		*paddr = LPAE_VTOP(vaddr);
+		if (!verbose)
+			return TRUE;
+	}
+
+
 	if (verbose)
 		fprintf(fp, "PAGE DIRECTORY: %lx\n", (ulong)pgd);
 
@@ -1221,6 +1273,11 @@ arm_kvtop(struct task_context *tc, ulong kvaddr, physaddr_t *paddr, int verbose)
 	if (!IS_KVADDR(kvaddr))
 		return FALSE;
 
+	if (machdep->flags & PAE)
+		return arm_lpae_vtop(kvaddr, (ulong *)vt->kernel_pgd[0],
+			paddr, verbose);
+
+
 	if (!vt->vmalloc_start) {
 		*paddr = VTOP(kvaddr);
 		return TRUE;
@@ -1232,9 +1289,6 @@ arm_kvtop(struct task_context *tc, ulong kvaddr, physaddr_t *paddr, int verbose)
 			return TRUE;
 	}
 
-	if (machdep->flags & PAE)
-		return arm_lpae_vtop(kvaddr, (ulong *)vt->kernel_pgd[0], 
-			paddr, verbose);
 
 	return arm_vtop(kvaddr, (ulong *)vt->kernel_pgd[0], paddr, verbose);
 }
@@ -1277,12 +1331,13 @@ arm_get_dumpfile_stack_frame(struct bt_info *bt, ulong *nip, ulong *ksp)
 {
 	const struct machine_specific *ms = machdep->machspec;
 
-	if (!ms->crash_task_regs)
+	if (!ms->crash_task_regs ||
+	    (!ms->crash_task_regs[bt->tc->processor].ARM_pc &&
+	     !ms->crash_task_regs[bt->tc->processor].ARM_sp)) {
+		bt->flags |= BT_REGS_NOT_FOUND;
 		return FALSE;
+	}
 
-	if (!is_task_active(bt->task))
-		return FALSE;
-	
 	/*
 	 * We got registers for panic task from crash_notes. Just return them.
 	 */
@@ -1315,11 +1370,9 @@ arm_get_stack_frame(struct bt_info *bt, ulong *pcp, ulong *spp)
 	else
 		ret = arm_get_frame(bt, &ip, &sp);
 
-	if (!ret) {
+	if (!ret)
 		error(WARNING, "cannot determine starting stack frame for task %lx\n",
 			bt->task);
-		return;
-	}
 
 	if (pcp)
 		*pcp = ip;
@@ -1486,6 +1539,7 @@ arm_dump_backtrace_entry(struct bt_info *bt, int level, ulong from, ulong sp)
 static ulong
 arm_vmalloc_start(void)
 {
+	machdep->machspec->vmalloc_start_addr = vt->high_memory;
 	return vt->high_memory;
 }
 
@@ -1632,7 +1686,12 @@ arm_display_machine_stats(void)
 static int
 arm_get_smp_cpus(void)
 {
-	return MAX(get_cpus_active(), get_cpus_online());
+	int cpus;
+	
+	if ((cpus = get_cpus_present()))
+		return cpus;
+	else
+		return MAX(get_cpus_online(), get_highest_cpu_online()+1);
 }
 
 /*
